/*
Rabih Kaddour
Copyright 2016! Inc. All rights reserved.
Licensed under the BSD License.
http://kaddour-es.de
*/

////////////////////////////
////// HELPERS
////////////////////////////
var cubes = [];  // just an array we can use to rotate the cubes
var renderer = null;
var scene = null;
var camera = null;
var canvas = null;
var fov = 45;
var near = 0.1;
var far = 1000;

function init() {
  // Get the document with Id
  var elmtDiv = document.getElementById('WebGLCanvas');
  if(elmtDiv) {
    canvas = elmtDiv;
    canvas.height = elmtDiv.clientHeight;
    canvas.width = elmtDiv.clientWidth;
  }
  else {
    canvas = document.getElementById('WebGLCanvas');
    canvas.width = window.innerWidth;
    canvas.height = window.innerHeight;
  }

  // Create a camera
  // Set a Field of View (FOV) of 75 degrees
  // Set an Apsect Ratio of the inner width divided by the inner height of the window
  // Set the 'Near' distance at which the camera will start rendering scene objects to 2
  // Set the 'Far' (draw distance) at which objects will not be rendered to 1000
  camera = new THREE.PerspectiveCamera(fov, canvas.width / canvas.height, near, far);

  // Set the camera position so that it's up top and looking down:
  var posX=200, posY=200, posZ=250;
  var deg = 90 * Math.PI / 180; // degInRad
  camera.lookAt(new THREE.Vector3(0,0,0));
  camera.rotateOnAxis(new THREE.Vector3(1, 0, 0), (deg));
  camera.up = new THREE.Vector3(0, 0, 1);

  // Move the camera 'out' by 200, 200 and 250
  camera.position.set(posX, posY, posZ);

  // Create main scene
  // This is the scene we will add all objects to.
  // Create a scene, that will hold all our elements such as objects, cameras and lights.
  scene = new THREE.Scene();

  // White directional light at half intensity shining from the top.
  var directionalLight = new THREE.DirectionalLight( 0xffffff, 1 );
  scene.add( directionalLight );

  // Add the camera to the scene
  scene.add(camera);

  // Create a cube
  var cubeGeo = new THREE.BoxGeometry(77, 77, 77);
  // var cubeMat = new THREE.MeshLambertMaterial({color: 0xff3300});

  // create a texture loader.
  var loader = new THREE.TextureLoader();
  // Load a texture.
  var cubeMat = new THREE.MeshBasicMaterial({
    map: loader.load("uv_test_bw_1024.png"),
  });
  // set the "color space" of the texture
  cubeMat.encoding = THREE.sRGBEncoding;
  // reduce blurring at glancing angles
  cubeMat.anisotropy = 16;

  var cube = new THREE.Mesh(cubeGeo, cubeMat);
  cube.position.set(0,0,0);

  // Add the cube to the scene
  scene.add(cube);

  // Add the cube to the array
  cubes.push(cube);

  camera.lookAt(scene.position);

  // Create a WebGL Rendered
  renderer = new THREE.WebGLRenderer();

  // Set the size of the rendered to the inner width and inner height of the window
  renderer.setPixelRatio( window.devicePixelRatio );
  renderer.setSize(canvas.width, canvas.height);

  // Background
  renderer.setClearColor(0xdddddd); // gray

  // Add in the created DOM element to the body of the document
  canvas.appendChild(renderer.domElement);

  // callback rendering
  // render();
  requestAnimationFrame(render);

  // Register an event listener to
  // call the resizeCanvas() function each time
  // the window is resized.
  window.addEventListener('resize', resizeRendererToDisplaySize, false);
}

// Animate the scene
function render(time) {
    // Render the scene
    renderScene(time);

    // This function calls itself on every frame. You can for example change
    // the objects rotation on every call to create a turntable animation.
    requestAnimationFrame( render );
}

// var clock = new THREE.Clock();
function renderScene(time) {
  time *= 0.001;

  cubes.forEach((cube, ndx) => {
    const speed = .2 + ndx * .1;
    const rot = time * speed;
    cube.rotation.x = rot;
    cube.rotation.y = rot;
  });

	renderer.render( scene, camera );
};

// Runs each time the DOM window resize event fires.
// Resets the canvas dimensions to match window,
// then draws the new borders accordingly.
function resizeRendererToDisplaySize() {
    // Check if the canvas is not the same size.
    var elmtDiv = document.getElementById('WebGLCanvas'); // img with id
    if(elmtDiv)
    {
    	canvas = elmtDiv;
    	canvas.height = elmtDiv.clientHeight;
    	canvas.width = elmtDiv.clientWidth;
    }
    if(camera != null)
    {
        camera.aspect = canvas.width / canvas.height;
        camera.updateProjectionMatrix();
    }
    if(renderer != null)
        renderer.setSize( canvas.width, canvas.height );
}
